#
# R script for exploiting the meta-analysis model.
# (S. S. Maier; June 2024)
#

# Clear memory ...
rm(list=ls(all.names=TRUE))
# Clear the console ...
cat("\014")  # or cat("\f")

# Install and launch the required R packages.
if(!require(openxlsx2)){
                        install.packages("openxlsx2")
                        library(openxlsx2)
                       }

if(!require(metafor)){
                      install.packages("metafor")
                      library(metafor)
                     }


cat(paste0("\n", "\033[1;", col=34, "m", "  ________________ Start ________________", "\033[0m", "\n", "\n"))

# Load the files that include data and model.
cat(paste0("\n", "\033[0;", col=32, "m","Load the files that include data and model ...","\033[0m","\n"))
cat(paste0("\033[0;", col=31, "m","          - Data.selected file ...","\033[0m","\n"))
Data.selected <- dget(file="Data.selected.txt")
cat(paste0("\033[0;", col=31, "m","          - model object file ...","\033[0m","\n"))
model <- dget(file="Example.model.txt")

# List the model structure.
cat(paste0("\n", "\033[0;", col=32, "m","List the model structure ...","\033[0m","\n"))
model.struct <- data.frame(cbind(Data.selected$Acronym,
                                 model.matrix(model),
                                 round(Data.selected$yi, 4),
                                 round(Data.selected$vi, 4)))
colnames(model.struct)[1] <- "Acronym"
colnames(model.struct)[8] <- "yi"
colnames(model.struct)[9] <- "vi"
print(model.struct)

# List the model details.
cat(paste0("\n", "\033[0;", col=32, "m", "List the model details ...","\033[0m", "\n"))
print(summary(model))

# List the model coefficients.
cat(paste0("\n", "\033[0;", col=32, "m", "List the model coefficients ...","\033[0m", "\n"))
print(cbind(coefficients(model)))

# Diagnose the model coefficients (t test and confidence intervals).
cat(paste0("\n", "\033[0;", col=32, "m","Diagnose the model coefficients ...","\033[0m","\n"))
cat(paste0("\033[0;", col=31, "m","          - t test ...","\033[0m","\n"))
print(
  clubSandwich::coef_test(obj=model,
                          cluster=Data.selected$NrDoses,
                          vcov = "CR2",  # or "CR3"
                          test = "Satterthwaite")
)

cat(paste0("\n", "\033[0;", col=31, "m","          - confidence intervals ...","\033[0m","\n"))
print(
  clubSandwich::conf_int(obj=model,
                         cluster=Data.selected$NrDoses,
                         vcov = "CR2",  # or "CR3"
                         test = "Satterthwaite")
)

# Extend the model diagnosis (for all model parameters).
cat(paste0("\n", "\033[0;", col=32, "m","Compute confidence intervals of the model parameters  ...","\033[0m","\n"))
cat("\033[1;35;mWait patiently for the result ...\033[0m\n")
print(metafor::confint.rma.mv(model))
cat(paste0("\n", "\033[0;", col=32, "m","How well-performing was the model fitting ... ?","\033[0m","\n"))
print(metafor::fitstats(model))

# Verify the model prediction ability / accuracy.
cat(paste0("\n", "\033[0;", col=32, "m","Verify the model prediction ability / accuracy ...","\033[0m","\n"))
print(metafor::predict.rma(model))

# Prepare and conduct the forest plot graphical diagnosis.
cat(paste0("\n", "\033[0;", col=32, "m","Prepare and conduct the forest plot graphical diagnosis ...","\033[0m","\n"))
Moderator <- "Start.score + Age + UnitDose"
InnerFact <- "UnitDose"
OuterFact <- "NrDoses"
Title <- paste0("Forest plot _ Model _ Mod=", Moderator, " _ Inner=", InnerFact, " _ Outer=", OuterFact, " _ Fitted CI", " _ Outliers removed")
tiff(filename=paste0(Title, ".tiff"), units="mm", width = 120, height = 75, res=1200, pointsize=6, compression = "lzw")

metafor::forest(model,
                header=TRUE,
                addfit=TRUE,
                addpred=FALSE,
                xlim=c(-5, 2.5),
                ylim=c(-0.5, length(Data.selected$Acronym)+3),
                refline=NA,
                slab = Acronym,
                ilab=Alternative,
                ilab.xpos=-3.5, ilab.pos=4,
                mlab="RE Model",
                colout=c(rep("navyblue", length(yi))),
                col = "lightgray", border="darkgray",
                xlab= "Effect size of the studies", cex.lab=1.2,
                main="Mixed random-effects model, with moderators (outliers removed)",
                shade="zebra", colshade="gray94"
               )
### Add Q-value, dfs, p-value, I^2, and tau2 statistic.
text(x=-5, y=-0.4, pos=4, cex=1,
     bquote(bold(paste("Summary: ( QE = ", .(formatC(model$QE, digits=2, format="f")),
                       ", df = ", .(model$QEdf),
                       ", p = ",  .(formatC(model$QEp, digits=4, format="f")),
                       "; QM = ", .(formatC(model$QM, digits=2, format="f")),
                       ", df = ", .(model$QMdf[[1]][1]),
                       ", p = ",  .(formatC(model$QMp, digits=4, format="f")),
                       "; ", tau^2, " = ", .(paste0("[", paste(formatC(model$tau2, digits=3, format="f"), collapse=", "), "]")),
                       " )"
                      )
                )
           ),
     col="darkred", adj=0)
### Add the zero position for effect sizes.
segments(x0=0, y0=0.5, x1=0, y1=length(Data.selected$Acronym)+1, col="black", lty="dotted")
text(x=0, y=0.25, labels="0", col="black", cex=0.8)
### Add the limits of large significant values for effect sizes.
segments(x0=-0.8, y0=0.5, x1=-0.8, y1=length(Data.selected$Acronym)+1, col="purple4", lty="dotted")
segments(x0= 0.8, y0=0.5, x1= 0.8, y1=length(Data.selected$Acronym)+1, col="darkgreen", lty="dotted")
text(x=-0.8, y=0.25, labels="-0.8 ", col="purple4", cex=0.8)
text(x= 0.8, y=0.25, labels= "0.8", col="darkgreen", cex=0.8)
### Add the MCID, minimal clinically important difference values for effect sizes.
# The shaded area denotes the area of clinical equivalence smaller than the
# minimal clinically important difference, MCID, on both sides.
segments(x0=-abs(model$MCID), y0=0.5, x1=-abs(model$MCID), y1=length(Data.selected$Acronym)+1, col="purple4", lty="dotted")
segments(x0= abs(model$MCID), y0=0.5, x1= abs(model$MCID), y1=length(Data.selected$Acronym)+1, col="darkgreen", lty="dotted")
text(x=-abs(model$MCID), y=0.25, labels=-abs(model$MCID), col="purple4", cex=0.8)
text(x= abs(model$MCID), y=0.25, labels= abs(model$MCID), col="darkgreen", cex=0.8)
rect(xleft=-abs(model$MCID), ybottom=0.5, xright=abs(model$MCID), ytop=length(Data.selected$Acronym)+1, col=adjustcolor("red", alpha.f = 0.1), border=adjustcolor("red", alpha.f = 0.1), xpd=FALSE)
rect(xleft=abs(model$MCID), ybottom=0.5, xright=0.8, ytop=length(Data.selected$Acronym)+1, col=adjustcolor("darkgreen", alpha.f = 0.1), border=adjustcolor("darkgreen", alpha.f = 0.1), xpd=FALSE)
rect(xleft=-0.8, ybottom=0.5, xright=-abs(model$MCID), ytop=length(Data.selected$Acronym)+1, col=adjustcolor("purple4", alpha.f = 0.1), border=adjustcolor("purple4", alpha.f = 0.1), xpd=FALSE)
# Add header details.
text(x=-3.5, y=length(Data.selected$Acronym) + 2.75, pos = 4,
     labels=substitute(paste(bold("Alternative treatment"))),
     cex=1)
text(x=-0.1, y=length(Data.selected$Acronym)+1.5, pos = 2,
     labels=substitute(paste(italic("        More favorable:\nAlternative treatment"))),
     col="purple4", cex=1)
text(x=0.1, y=length(Data.selected$Acronym)+1.5, pos = 2,
     labels="|\n|",
     cex=1)
text(x=0.1, y=length(Data.selected$Acronym)+1.5, pos = 4,
     labels=substitute(paste(italic("More favorable:\nHA treatment"))),
     col="darkgreen", cex=1)
### Add the closing line of the plot.
segments(x0=-5, y0=0.5, x1=2.5, y1=0.5, col="black", lty="solid")
dev.off()

# Create the bubble/scatter plot and print the regression line, according the model. Continous moderators only.
metafor::regplot(model, mod="Start.score", xlab="Start score (WOMAC equivalent)", col="darkred", bg="yellow", lcol=c("darkred", "darkblue"), label="all", offset=3, labsize=0.8, legend="topright", grid=TRUE, refline=0)
metafor::regplot(model, mod="Age", col="darkred", bg="yellow", lcol=c("darkred", "darkblue"), label="all", offset=3, labsize=0.8, legend=FALSE, grid=TRUE, refline=0)

# Prepare the trim and fill testing and funnel plot diagnosis. Load the simplified models models.
cat(paste0("\n", "\033[0;", col=32, "m","Prepare the trim and fill testing and funnel plot diagnosis ...","\033[0m","\n"))
cat(paste0("\033[0;", col=31, "m","          - Load the equal-effects model ...","\033[0m","\n"))
model.fix.EE  <- dget(file="Exmple.model.fix.EE.txt")
cat(paste0("\033[0;", col=31, "m","          - Load the random-effects model ...","\033[0m","\n"))
model.fix.REML <- dget(file="Exmple.model.fix.REML.txt")

# Testing the side that should be supplemented (filled)
cat(paste0("\n", "\033[0;", col=32, "m","Testing if and the side that should be supplemented with studies ...","\033[0m","\n"))
cat(paste0("\033[0;", col=31, "m","          - ... on the left side ...","\033[0m","\n"))
metafor::trimfill(x=model.fix.EE, side="left", estimator="R0")
cat(paste0("\033[0;", col=31, "m","          - ... on the right side ...","\033[0m","\n"))
metafor::trimfill(x=model.fix.EE, side="right", estimator="R0")

# Generate the trim-and-fill models.
cat(paste0("\n", "\033[0;", col=32, "m","Generate the trim-and-fill models ...","\033[0m","\n"))
cat(paste0("\033[0;", col=31, "m","          - The extended model of equal-effects type (right side) ...","\033[0m","\n"))
model.taf.EE <- metafor::trimfill(x=model.fix.EE, side="right", estimator="R0")
print(summary(model.taf.EE))
cat(paste0("\033[0;", col=31, "m","          - The points of the extended EE model (right side) ...","\033[0m","\n"))
taf.points.EE <- data.frame(yi = model.taf.EE$yi, vi = model.taf.EE$vi, fill = model.taf.EE$fill)
print(taf.points.EE)

cat(paste0("\n", "\033[0;", col=32, "m","Generate the trim-and-fill models ...","\033[0m","\n"))
cat(paste0("\033[0;", col=31, "m","          - The extended model of random-effects type (right side) ...","\033[0m","\n"))
model.taf.REML <- metafor::trimfill(x=model.fix.REML, side="right", estimator="R0")
print(summary(model.taf.REML))
cat(paste0("\033[0;", col=31, "m","          - The points of the extended RE model (right side) ...","\033[0m","\n"))
taf.points.REML <- data.frame(yi = model.taf.REML$yi, vi = model.taf.REML$vi, fill = model.taf.REML$fill)
print(taf.points.REML)


# Trim and fill plot _ Fitting precision.
cat(paste0("\n", "\033[0;", col=32, "m","Plot the trim-and-fill models ...","\033[0m","\n"))
cat(paste0("\033[0;", col=31, "m","          - the reciprocal of studies precision ...","\033[0m","\n"))
Title <- paste0("Funnel plot _ Trim and fill _ SEI _ Mod=", Moderator, " _ Inner=", InnerFact, " _ Outer=", OuterFact, " _ Fitted CI", " _ Outlier removed")
tiff(filename=paste0(Title, ".tiff"), units="mm", width = 100, height = 90, res=1200, pointsize=6, compression = "lzw")
metafor::funnel(x=model.taf.EE,
                yaxis="sei", level=95,
                addtau2=TRUE,
                #back=adjustcolor("darkgreen", alpha.f = 0.1),
                #shade=adjustcolor("red", alpha.f = 0.1),
                back=adjustcolor("blue", alpha.f = 0.1),
                shade=adjustcolor("green", alpha.f = 0.1),
                pch=21,
                pch.fill=22,
                col=c("darkred", "darkblue"), bg=c("yellow", "cyan"), cex=1.2,
                label=TRUE, offset=0.3,
                main="Trim and fill plot (The reciprocal of studies precision)",
                legend="topleft")
abline(v=-abs(model$MCID), lty="dashed", col="purple4")
abline(v=abs(model$MCID), lty="dashed", col="purple4")
abline(v=-0.8, lty="dashed", col="darkgreen")
abline(v=0.8, lty="dashed", col="darkgreen")
text(x=model.taf.EE$beta, y=-0.005, labels=round(model.taf.EE$beta[1], 2), cex=0.8)
text(x=-abs(model$MCID), y=0.01, labels="             -0.37 ", col="purple4", cex=0.8)
text(x=-0.8,  y=0.01, labels="-0.8         ", col="darkgreen", cex=0.8)
text(x=abs(model$MCID), y=0.01, labels="0.37              ", col="purple4", cex=0.8)
text(x=0.8,  y=0.01, labels="         0.8", col="darkgreen", cex=0.8)
dev.off()


# Trim and fill plot _ Fitting precision _ SE + tau2.
cat(paste0("\033[0;", col=31, "m","          - the tau2 extended reciprocal of studies precision ...","\033[0m","\n"))
Title <- paste0("Funnel plot _ Trim and fill _ SEI + tau2 _ Mod=", Moderator, " _ Inner=", InnerFact, " _ Outer=", OuterFact, " _ Fitted CI", " _ Outlier removed")
tiff(filename=paste0(Title, ".tiff"), units="mm", width = 100, height = 90, res=1200, pointsize=6, compression = "lzw")
metafor::funnel(x=model.taf.REML,
                yaxis="sei", level=95,
                addtau2=TRUE,
                back=adjustcolor("blue", alpha.f = 0.1),
                shade=adjustcolor("green", alpha.f = 0.1),
                pch=21,
                pch.fill=22,
                col=c("darkred", "darkblue"), bg=c("yellow", "cyan"), cex=1.2,
                label=TRUE, offset=0.3,
                main="Trim and fill plot (The reciprocal of studies precision)",
                legend="topleft")
abline(v=-abs(model$MCID), lty="dashed", col="purple4")
abline(v=abs(model$MCID), lty="dashed", col="purple4")
abline(v=-0.8, lty="dashed", col="darkgreen")
abline(v=0.8, lty="dashed", col="darkgreen")
text(x=model.taf.REML$beta, y=-0.005, labels=round(model.taf.REML$beta[1], 2), cex=0.8)
text(x=-abs(model$MCID), y=0.01, labels="             -0.37 ", col="purple4", cex=0.8)
text(x=-0.8,  y=0.01, labels="-0.8         ", col="darkgreen", cex=0.8)
text(x=abs(model$MCID), y=0.01, labels="0.37              ", col="purple4", cex=0.8)
text(x=0.8,  y=0.01, labels="         0.8", col="darkgreen", cex=0.8)
dev.off()


# Trim and fill plot _ Robustness.
cat(paste0("\033[0;", col=31, "m","          - the robustness of studies ...","\033[0m","\n"))
Title <- paste0("Funnel plot _ Trim and fill _ NI _ Mod=", Moderator, " _ Inner=", InnerFact, " _ Outer=", OuterFact, " _ Fitted CI", " _ Outlier removed")
tiff(filename=paste0(Title, ".tiff"), units="mm", width = 100, height = 90, res=1200, pointsize=6, compression = "lzw")
metafor::funnel(x=model.taf.EE,
                yaxis="ni", level=95,
                back=adjustcolor("blue", alpha.f = 0.1),
                pch=21,
                pch.fill=22,
                col=c("darkred", "darkblue"), bg=c("yellow", "cyan"), cex=1.2,
                label=TRUE, offset=0.3,
                main="Trim and fill plot (The size of the patient sample)",
                legend="topleft")
abline(v=-abs(model$MCID), lty="dashed", col="purple4")
abline(v=abs(model$MCID), lty="dashed", col="purple4")
abline(v=-0.8, lty="dashed", col="darkgreen")
abline(v=0.8, lty="dashed", col="darkgreen")
text(x=model.taf.EE$beta - 0.12, y=175, labels=round(model.fix.EE$beta[1], 2), cex=0.8)
text(x=-abs(model$MCID), y=175, labels="             -0.37 ", col="purple4", cex=0.8)
text(x=-0.8,  y=175, labels="-0.8         ", col="darkgreen", cex=0.8)
text(x=abs(model$MCID), y=175, labels="0.37              ", col="purple4", cex=0.8)
text(x=0.8,  y=175, labels="         0.8", col="darkgreen", cex=0.8)
dev.off()


# Trim and fill plot _ Robustness _ SE + tau2.
cat(paste0("\033[0;", col=31, "m","          - the tau2 extended robustness of studies ...","\033[0m","\n"))
Title <- paste0("Funnel plot _ Trim and fill _ NI + tau2 _ Mod=", Moderator, " _ Inner=", InnerFact, " _ Outer=", OuterFact, " _ Fitted CI", " _ Outlier removed")
tiff(filename=paste0(Title, ".tiff"), units="mm", width = 100, height = 90, res=1200, pointsize=6, compression = "lzw")
metafor::funnel(x=model.taf.REML,
                yaxis="ni", level=95,
                back=adjustcolor("blue", alpha.f = 0.1),
                pch=21,
                pch.fill=22,
                col=c("darkred", "darkblue"), bg=c("yellow", "cyan"), cex=1.2,
                label=TRUE, offset=0.3,
                main="Trim and fill plot (The size of the patient sample)",
                legend="topleft")
abline(v=-abs(model$MCID), lty="dashed", col="purple4")
abline(v=abs(model$MCID), lty="dashed", col="purple4")
abline(v=-0.8, lty="dashed", col="darkgreen")
abline(v=0.8, lty="dashed", col="darkgreen")
text(x=model.taf.REML$beta - 0.12, y=175, labels=round(model.taf.REML$beta[1], 2), cex=0.8)
text(x=-abs(model$MCID), y=175, labels="             -0.37 ", col="purple4", cex=0.8)
text(x=-0.8,  y=175, labels="-0.8         ", col="darkgreen", cex=0.8)
text(x=abs(model$MCID), y=175, labels="0.37              ", col="purple4", cex=0.8)
text(x=0.8,  y=175, labels="         0.8", col="darkgreen", cex=0.8)
dev.off()
